<?php

	/*
	*	Gekko - Open Source Web Development Framework
	*	------------------------------------------------------------------------
	*	Copyright (C) 2004-2006, J. Carlos Nieto <xiam@users.sourceforge.net>
	*	This program is Free Software.
	*
	*	@package	Gekko
	*	@license	http://www.gnu.org/copyleft/gpl.html GNU/GPL License 2.0
	*	@author		J. Carlos Nieto <xiam@users.sourceforge.net>
	*	@version	CVS: $Id: main.lib.php,v 1.1.1.1 2006/02/20 21:52:23 xiam Exp $
	*	@link		http://www.gekkoware.org
	*/

	if (!defined("IN-GEKKO")) die("Get a life!");

	Class Forums {
		/**
		* messageAuthEdit(message_id, creation_time, author_id);
		* @returns: true if the user owns the right to edit this post (he created
		* it and is within edition time)
		*/
		function messageAuthEdit($id, $creation, $authorid) {
			return (gauth("forums") || (gauth("$") && ($authorid == $GLOBALS["USER"]["id"]) && ($creation+conf::getKey("forums", "edit_time")) >= time()));
		}

		/**
		* mkuserlink(user_id, alternate_nickname, alternate_url);
		* @returns: HTML Formatted user link
		*/
		function mkUserLink(&$row) {
			if (!gauth("forums") && conf::getkey("forums", "hide_ip_and_email", 1, "b")) {
				$row["contactmail"] = "";
			}
			return ($author = Users::mkUserLink($row["userid"])) ? $author : "&lt;".safeLink($row["contactmail"], $row["nickname"] ? $row["nickname"] : _l("L_ANONYMOUS"))."&gt;";
		}

		/**
		* deleteThread([thread_id);
		* Completely removes a thread an its associated messages
		*/
		function deleteThread($id) {
			$db =& $GLOBALS["db"];
			$db->query("DELETE FROM ".db_table("forum_thread")." WHERE id = '{$id}'", __FILE__, __LINE__, true);
			$db->query("DELETE FROM ".db_table("forum_message")." WHERE topicid = '{$id}'", __FILE__, __LINE__, true);
			slog("forums: deleted forum thread '$id'", "actions", 1);
		}

		/**
		* deleteForum(forum_id);
		* Completely removes a forum and all its associated threads
		*/
		function deleteForum($id) {
			$db =& $GLOBALS["db"];

			$db->query("DELETE FROM ".db_table("forum_forum")." WHERE id = '{$id}'", __FILE__, __LINE__, true);

			$q = $db->query("SELECT id FROM ".db_table("forum_thread")."
			WHERE forumid = '{$id}'", __FILE__, __LINE__, true);

			while ($row = $db->fetch_row($q))
				forums::deleteThread($row["id"]);

			slog("forums: deleted forum '$id'", "actions", 1);
		}

		/**
		* deleteItem(category_id);
		* Removes a category and all its associated forums
		*/
		function deleteItem($id) {
			$db =& $GLOBALS["db"];

			$db->query("DELETE FROM ".db_table("forum_category")." WHERE id = '{$id}'", __FILE__, __LINE__, true);

			$q = $db->query("SELECT id FROM ".db_table("forum_forum")."
			WHERE categoryid = '{$id}'", __FILE__, __LINE__, true);

			while ($row = $db->fetch_row($q))
				forums::deleteForum($row["id"]);

			slog("forums: deleted category '$id'", "actions", 1);
		}

		/**
		* mkStatusIcon([is_sticked], [is_locked]);
		* @returns: HTML formatted status icon for ilustrating the forum
		*/
		function mkStatusIcon($sticked = false, $locked = false) {
			if ($sticked && $locked) {
				$icon = "sticked-locked";
			} elseif ($sticked) {
				$icon = "stick";
			} elseif ($locked) {
				$icon = "lock";
			} else {
				$icon = "open";
			}
			return mkIcon("$icon.png", 16);
		}

		/**
		* updateForumStats(forum_id);
		* Updates statistics for specified forum such as number of topics, messages, last message, etc
		*/
		function updateForumStats($forum) {
			$db =& $GLOBALS["db"];

			// null counters
			$topics = $messages = $last_message = 0;

			$q = $db->query("SELECT id, stat_replies FROM ".db_table("forum_thread")."
			WHERE forumid = '{$forum}'", __FILE__, __LINE__, true);

			// how many topics in this thread?
			$topics = $db->num_rows($q);

			if ($topics) {
				while ($row = $db->fetch_row($q)) {
					// replies
					$messages += $row["stat_replies"];
				}

				// last message id
				$q = $db->query("SELECT m.id FROM ".db_table("forum_message")." m,
				".db_table("forum_thread")." t
				WHERE t.forumid = '{$forum}' AND m.topicid = t.id
				ORDER BY m.creationtime DESC LIMIT 1
				", __FILE__, __LINE__, true);

				$row = $db->fetch_row($q);
				$last_message = $row["id"];
			}

			$q = $db->query("
				UPDATE ".db_table("forum_forum")." SET
					messagecount = '$messages',
					topiccount = '$topics',
					lastpostid = '$last_message'
				WHERE id = '{$forum}'
			", __FILE__, __LINE__, true);

		}
		/**
		* updateThreadStats(thread_id, [do_read]);
		* Updates statistics for a forum thread such as number of reads
		*/
		function updateThreadStats($thread, $read = false) {
			$db =& $GLOBALS["db"];

			// number of messages in this thread
			$q = $db->query("SELECT id FROM ".db_table("forum_message")."
			WHERE topicid = '{$thread}' AND is_topic = '0'
			", __FILE__, __LINE__, true);

			$replies = $db->num_rows($q);

			$db->query("UPDATE ".db_table("forum_thread")." SET
			stat_replies = '{$replies}' ".($read ? ", stat_reads = stat_reads+1": "")."
			WHERE id = '{$thread}'", __FILE__, __LINE__,
			true);
		}
		/**
		* checkAuth(category_id, forum_id, thread_id);
		* @returns: an array of permissions to allow or deny the user to post, reply or view a thread, category or forum
		*/
		function checkAuth($cat_id, $forum_id = 0, $thread_id = 0) {

			$db =& $GLOBALS["db"];

			$auth = Array (
				"view"	=> false,
				"post"	=> false,
				"reply"	=> false
			);

			if (gauth("forums")) {

				// administrator is god :)
				$auth["view"] = $auth["post"] = $auth["reply"] = true;

			} else {

				$allowed = forums::getAllowedCategories();

				// checking access for category
				$q = $db->query("SELECT auth_access, auth_post FROM
				".db_table("forum_category")." WHERE id = '{$cat_id}'", __FILE__, __LINE__, true);

				if ($db->num_rows($q)) {
					$row = $db->fetch_row($q);
					$auth["view"]  = gauth($row["auth_access"]);
					$auth["post"] = $auth["reply"] = gauth($row["auth_post"]);
				} else {
					// this category doesn't exists!
					$auth["view"] = $auth["post"] = $auth["reply"] = false;
					return $auth;
				}


				// checking access for forum
				if ($forum_id) {
					$q = $db->query("SELECT id
					FROM ".db_table("forum_forum")." WHERE id = '{$forum_id}'",
					__FILE__, __LINE__, true);

					if ($db->num_rows($q)) {
						$auth["reply"] = $auth["post"];
					} else {
						$auth["post"] = $auth["view"] = $auth["reply"] = false;
						return $auth;
					}
				}

				// checking access for thread
				if ($thread_id) {

					$q = $db->query("SELECT id, mode_locked
					FROM ".db_table("forum_thread")."
					WHERE id = '{$thread_id}'", __FILE__, __LINE__, true);

					if ($db->num_rows($q) && $auth["view"]) {
						$row = $db->fetch_row($q);

						$auth["post"] = $auth["reply"] = (!$row["mode_locked"]);

					} else {
						// thread doesnt exists!
						$auth["view"] = $auth["post"] = $auth["reply"] = false;
						return $auth;
					}
				}


			}
			return $auth;

		}
		/**
		* getForums();
		* @returns:  an array containing a list of forums
		*/
		function getForums() {

			$db =& $GLOBALS["db"];

			$q = $db->query("SELECT id, name
			FROM ".db_table("forum_forum")."
			ORDER BY categoryid DESC
			", __FILE__, __LINE__, true);

			$forums = Array();
			while ($row = $db->fetch_row($q))
				$forums[$row["id"]] = $row["name"];

			return $forums;
		}
		/**
		* getAllowedCategories([create_sql_lock], [sql_lock_field], [auth]);
		*/
		function getAllowedCategories($mklock = false, $lockfield = "categoryid", $auth = null) {
			$db =& $GLOBALS["db"];

			$allowed = Array();

			$q = $db->query("SELECT id, auth_access
			FROM ".db_table("forum_category")." WHERE status = '1'",
			__FILE__, __LINE__, true);

			while ($row = $db->fetch_row($q))
				if (gauth($row["auth_access"], $auth))
					$allowed[] = $row["id"];

			if ($mklock)
				$allowed = "$lockfield = '".implode($allowed, "' OR $lockfield = '")."'";

			return $allowed;
		}

		/**
		* allowedCategories();
		* @returns: array of all forum categories
		*/
		function listCategories() {
			$r = array();
			$db =& $GLOBALS["db"];

			$q = $db->query("SELECT id, name FROM ".db_table("forum_category")."
			ORDER BY orderid ASC",
			__FILE__, __LINE__, true);

			while ($row = $db->fetch_row($q)) {
				saneHTML($row);
				$r[$row["id"]] = $row["name"];
			}
			return $r;
		}
	}
?>